/*
    env.c

    This file is a part of Anubis.
    Copyright (C) 2001, 2002 The Anubis Team.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "headers.h"
#include "getopt.h"
#include "extern.h"

static int gindex = 0;
static struct option gopt[] =
{
	{"host",			1, 0, 'h'},
	{"tunnel",			1, 0, 't'},
	{"local-mta",		1, 0, 'l'},
	{"foreground",		0, 0, 'f'},
	{"sio",				0, 0, 'i'},
	{"auth-local",		0, 0, 0},
	{"ident-only",		0, 0, 0},
	#ifndef NOFORK
	{"setremoteuser",	0, 0, 0},
	#endif /* not NOFORK */
	{"socks-proxy",		1, 0, 0},
	{"socks-v4",		0, 0, 0},
	{"socks-username",	1, 0, 0},
	{"socks-password",	1, 0, 0},
	{"wingate1",		1, 0, 0},
	{"wingate2",		1, 0, 0},
	#ifndef NOSSL
	{"nossl",			0, 0, 0},
	{"ssl-only",		0, 0, 0},
	{"cert",			1, 0, 0},
	{"key",				1, 0, 0},
	#endif /* not NOSSL */
	#ifndef TUNNEL_OPTIMIZE
	{"from",			1, 0, 'F'},
	{"reply-to",		1, 0, 'R'},
	{"to",				1, 0, 'T'},
	{"cc",				1, 0, 0},
	{"bcc",				1, 0, 0},
	{"subject",			1, 0, 'S'},
	{"body",			1, 0, 'B'},
	{"body-file",		1, 0, 0},
	{"realuser",		0, 0, 'r'},
	{"noroot",			0, 0, 0},
	{"smtp-auth",		0, 0, 0},
	{"smtp-mode",		1, 0, 0},
	#ifndef NOGPG
	{"pgp-sign",		0, 0, 0},
	{"pgp-encrypt",		1, 0, 0},
	{"rm-pgp",			1, 0, 0},
	#endif /* not NOGPG */
	{"rm-pgp-file",		1, 0, 0},
	{"rm-rrt",			1, 0, 0},
	{"rm-post",			1, 0, 0},
	{"rm-header",		1, 0, 0},
	{"rm-lt",			1, 0, 0},
	{"rm-rlt",			0, 0, 0},
	{"mix",				0, 0, 0},
	{"file",			1, 0, 0},
	{"uuencode",		0, 0, 0},
	{"extra-h1",		1, 0, 0},
	{"extra-h2",		1, 0, 0},
	{"overload-helo",	0, 0, 0},
	{"rot13",			0, 0, 0},
	{"rot13-subject",	0, 0, 0},
	{"rot13-body",		0, 0, 0},
	{"mid",				1, 0, 0},
	{"xmailer",			1, 0, 0},
	{"xmailer-no",		0, 0, 0},
	{"charset",			1, 0, 0},
	{"fake-no",			0, 0, 0},
	{"fake-ip1",		1, 0, 0},
	{"fake-ip2",		1, 0, 0},
	{"fake-host",		1, 0, 0},
	{"signature",		0, 0, 0},
	#endif /* not TUNNEL_OPTIMIZE */
	{"silent",			0, 0, 's'},
	{"verbose",			0, 0, 'v'},
	#ifdef DEBUG_CODE
	{"debug",			0, 0, 'D'},
	#endif /* DEBUG_CODE */
	#ifndef NOPRI
	{"pri",				1, 0, 0},
	#endif /* not NOPRI */
	#ifndef NOANSI
	{"noansi",			0, 0, 0},
	#endif /* not NOANSI */
	{"version",			0, 0, 0},
	{"help",			0, 0, 0},
	{"altrc",			1, 0, 0},
	{"norc",			0, 0, 0},
	{0, 0, 0, 0}
};

void get_options(int argc, char *argv[])
{
	int c;

	while ((c = getopt_long(argc, argv, "h:t:l:F:T:R:S:B:irfsvD?",
	gopt, &gindex)) != EOF)
	{
		switch (c)
		{
			case 0:
				if (strcmp(gopt[gindex].name, "help") == 0)
					print_usage();
				if (strcmp(gopt[gindex].name, "version") == 0)
					print_version();

				if (strcmp(gopt[gindex].name, "norc") == 0)
					popt |= P_NORC;
				if (strcmp(gopt[gindex].name, "altrc") == 0) {
					altrc = optarg;
					popt |= P_ALTRC;
				}
				if (strcmp(gopt[gindex].name, "auth-local") == 0)
					topt |= T_AUTH_LOCAL;
				if (strcmp(gopt[gindex].name, "ident-only") == 0)
					topt |= T_IDENT_ONLY;
				#ifndef NOFORK
				if (strcmp(gopt[gindex].name, "setremoteuser") == 0)
					topt |= T_SETREMOTE_USER;
				#endif /* not NOFORK */

				/*
				   SOCKS Proxy.
				*/

				if (strcmp(gopt[gindex].name, "socks-proxy") == 0) {
					parse_hostname_port(optarg, socks_proxy, &socks_port);
					popt |= P_SOCKS;
				}
				if (strcmp(gopt[gindex].name, "socks-v4") == 0)
					popt |= P_SOCKSV4;
				if (strcmp(gopt[gindex].name, "socks-username") == 0) {
					safe_strcpy(socks_username, optarg);
					popt |= P_SOCKSUSERNAME;
				}
				if (strcmp(gopt[gindex].name, "socks-password") == 0) {
					safe_strcpy(socks_password, optarg);
					popt |= P_SOCKSPASSWORD;
				}

				/*
				   WinGates.
				*/

				if (strcmp(gopt[gindex].name, "wingate1") == 0) {
					parse_hostname_port(optarg, wingate1, &wingate1_port);
					popt |= P_WG1;
				}
				if (strcmp(gopt[gindex].name, "wingate2") == 0) {
					parse_hostname_port(optarg, wingate2, &wingate2_port);
					popt |= P_WG2;
				}

				/*
				   TLS/SSL support.
				*/

				#ifndef NOSSL
				if (strcmp(gopt[gindex].name, "nossl") == 0)
					popt |= P_SSL_NO;
				if (strcmp(gopt[gindex].name, "ssl-only") == 0)
					popt |= P_SSL_ONLY;
				if (strcmp(gopt[gindex].name, "cert") == 0) {
					xfree(ssl_cert);
					ssl_cert = allocbuf(optarg, MAXPATHLEN);
				}
				if (strcmp(gopt[gindex].name, "key") == 0) {
					xfree(ssl_key);
					ssl_key = allocbuf(optarg, MAXPATHLEN);
				}
				#endif /* not NOSSL */

				#ifndef TUNNEL_OPTIMIZE

				/*
				   ESMTP AUTH.
				*/

				if (strcmp(gopt[gindex].name, "smtp-auth") == 0)
					popt |= P_SMTPAUTH;

				/*
				    GnuPG support.
				*/

				#ifndef NOGPG
				if (strcmp(gopt[gindex].name, "pgp-sign") == 0)
					mopt |= M_PGPSIGN;
				if (strcmp(gopt[gindex].name, "pgp-encrypt") == 0) {
					pgp_keys = allocbuf(optarg, 0);
					mopt |= M_PGP;
				}
				#endif /* not NOGPG */

				/*
				   REAL STEALTH MODE V2... ;-)
				*/

				if (strcmp(gopt[gindex].name, "fake-no") == 0)
					popt |= P_FAKENO;
				if (strcmp(gopt[gindex].name, "fake-ip1") == 0) {
					safe_strcpy(fakeip1, optarg);
					popt |= P_FAKEIP1;
				}
				if (strcmp(gopt[gindex].name, "fake-ip2") == 0) {
					safe_strcpy(fakeip2, optarg);
					popt |= P_FAKEIP2;
				}
				if (strcmp(gopt[gindex].name, "fake-host") == 0) {
					fakehost = allocbuf(optarg, 65);
					popt |= P_FAKEHOST;
				}

				/*
				   Remailer Type-I support.
				*/

				if (strcmp(gopt[gindex].name, "rm-rrt") == 0) {
					rm_rrt = allocbuf(optarg, 0);
					mopt |= M_RM;
					mopt |= M_RMRRT;
				}
				if (strcmp(gopt[gindex].name, "rm-post") == 0) {
					rm_post = allocbuf(optarg, 0);
					mopt |= M_RM;
					mopt |= M_RMPOST;
				}
				#ifndef NOGPG
				if (strcmp(gopt[gindex].name, "rm-pgp") == 0) {
					rm_pgp_key = allocbuf(optarg, 0);
					mopt |= M_RMPGP;
				}
				#endif /* not NOGPG */

				if (strcmp(gopt[gindex].name, "rm-pgp-file") == 0) {
					rm_pgp_file = allocbuf(optarg, MAXPATHLEN);
					mopt |= M_RMPGPFILE;
				}
				if (strcmp(gopt[gindex].name, "rm-header") == 0) {
					rm_header = allocbuf(optarg, 512);
					mopt |= M_RMHEADER;
				}
				if (strcmp(gopt[gindex].name, "rm-lt") == 0) {
					safe_strcpy(rm_lt, optarg);
					mopt |= M_RMLT;
				}
				if (strcmp(gopt[gindex].name, "rm-rlt") == 0)
					mopt |= M_RMRLT;

				/*
				   Remailer Type-II support (The Mixmaster).
				*/

				if (strcmp(gopt[gindex].name, "mix") == 0)
					mopt |= M_MIXMASTER;

				/*
				   Mail options.
				*/

				if (strcmp(gopt[gindex].name, "cc") == 0) {
					carbon_copy = allocbuf(optarg, 0);
					mopt |= M_CC;
				}
				if (strcmp(gopt[gindex].name, "bcc") == 0) {
					blind_carbon_copy = allocbuf(optarg, 0);
					mopt |= M_BCC;
				}
                		if (strcmp(gopt[gindex].name, "mid") == 0) {
					messageid = allocbuf(optarg, 256);
					mopt |= M_MID_CUSTOM;
				}
				if (strcmp(gopt[gindex].name, "xmailer") == 0) {
					xmailer = allocbuf(optarg, 256);
					mopt |= M_XMAIL_CUSTOM;
				}
				if (strcmp(gopt[gindex].name, "xmailer-no") == 0)
					mopt |= M_XMAIL_NO;
				if (strcmp(gopt[gindex].name, "noroot") == 0)
					mopt |= M_NOROOT;
                		if (strcmp(gopt[gindex].name, "overload-helo") == 0) {
					popt |= P_OVERLOAD;
				}
                		if (strcmp(gopt[gindex].name, "charset") == 0) {
					safe_strcpy(charset, optarg);
					mopt |= M_CHARSET;
				}
				if (strcmp(gopt[gindex].name, "extra-h1") == 0) {
					extra_h1 = allocbuf(optarg, 512);
					mopt |= M_EH1;
				}
				if (strcmp(gopt[gindex].name, "extra-h2") == 0) {
					extra_h2 = allocbuf(optarg, 512);
					mopt |= M_EH2;
				}
				if (strcmp(gopt[gindex].name, "body-file") == 0) {
					body_file = allocbuf(optarg, MAXPATHLEN);
					msgbody = (char *)xmalloc(1);
					mopt |= M_BODYATTACH;
					mopt |= M_BODY;
				}
				if (strcmp(gopt[gindex].name, "signature") == 0)
					mopt |= M_SIGNATURE;

				/*
				   Attach one file to the message.
				*/

				if (strcmp(gopt[gindex].name, "file") == 0) {
					path = allocbuf(optarg, MAXPATHLEN);
					mopt |= M_FILEATTACH;
				}
				if (strcmp(gopt[gindex].name, "uuencode") == 0)
					trans_ptr = uu_std; /* Uuencode instead of Base64 */

				/*
				   ROT-13 encoding.
				*/

				if (strcmp(gopt[gindex].name, "rot13") == 0)
					mopt |= M_ROT13;
				if (strcmp(gopt[gindex].name, "rot13-subject") == 0)
					mopt |= M_ROT13S;
				if (strcmp(gopt[gindex].name, "rot13-body") == 0)
					mopt |= M_ROT13B;

				/*
				   Misc. options.
				*/

				if (strcmp(gopt[gindex].name, "smtp-mode") == 0) {
					if ((strlen(optarg)) != 4) {
						mprintf("%s: Unknown SMTP mode. Try: MAIL, SEND, SOML, SAML.",
						argv[0]);
						quit(0);
					}
					safe_strcpy(smtp_mode, optarg);
				}

				#endif /* not TUNNEL_OPTIMIZE */

				#ifndef NOANSI
				if (strcmp(gopt[gindex].name, "noansi") == 0)
					popt |= P_NOANSI;
				#endif /* not NOANSI */

				#ifndef NOPRI
				if (strcmp(gopt[gindex].name, "pri") == 0) {
					int pri = atoi(optarg);
					if (pri > -21 && pri < 21)
						setpriority(PRIO_PROCESS, getpid(), pri);
					else {
						mprintf("%s: --pri : set task priority <-20,+20>", argv[0]);
						quit(EXIT_FAILURE);
					}
				}
				#endif /* not NOPRI */

				break;
			case 'h': /* a host name or IP address */
				parse_hostname_port(optarg, hostname, &portnumber);
				break;
			case 't': /* daemon's port number, host name */
				parse_hostname_host(optarg, tunnel_hostname, &tunnel_portnumber);
				popt &= ~P_SMTP;
				topt |= T_TUNNEL;
				if (strlen(tunnel_hostname) != 0)
					topt |= T_NAMES;
				break;
			case 'l': /* local mode */
				#ifndef NOFORK
				 execpath = allocbuf(optarg, MAXPATHLEN);
				 topt |= T_LOCAL_MTA;
				#else
				 mprintf("%s: Local mode not supported under this OS.", argv[0]);
				 quit(0);
				#endif /* not NOFORK */
				break;
			case 'f': /* foreground mode */
				popt |= P_FOREGROUND_INIT;
				break;
			case 'i': /* stdin/stdout */
				topt |= T_STD_INOUT;
				topt |= T_TUNNEL;
				popt &= ~P_SMTP;
				break;

			#ifndef TUNNEL_OPTIMIZE
			case 'r': /* Be a REAL user :) */
				popt |= P_REALUSER;
				break;
			case 'F': /* From: */
				from = allocbuf(optarg, 256);
				mopt |= M_FROM;
				break;
			case 'T': /* To: */
				to = allocbuf(optarg, 256);
				mopt |= M_TO;
				break;
			case 'R': /* Reply-To: */
				reply_to = allocbuf(optarg, 256);
				mopt |= M_REPLYTO;
				break;
			case 'S': /* Subject: */
				subject = allocbuf(optarg, 512);
				mopt |= M_SUBJECT;
				break;
			case 'B': /* Message body */
				msgbody = (char *)xmalloc(strlen(optarg) + 4);
				strncpy(msgbody, optarg, strlen(optarg));
				strncat(msgbody, CRLF, 2);
				mopt |= M_BODY;
				break;
			#endif /* not TUNNEL_OPTIMIZE */

			case 'v': /* verbose */
				output = VERBOSE;
				break;
			#ifdef DEBUG_CODE
			case 'D': /* debug */
				output = DEBUG;
				break;
			#endif /* DEBUG_CODE */
			case 's': /* silent */
				output = SILENT;
				break;
			case '?':
			default:
				mprintf("Try '%s --help' for more information.", argv[0]);
				quit(0);
		}
	}
	#ifndef NOFORK
	if (topt & T_LOCAL_MTA) {
		if (optind == argc) { /* No extra arguments specified. */
			if (execpath) {
				char *ptr = strrchr(execpath, '/');
				if (ptr)
					ptr++;
				execargs = gen_execargs(ptr);
				topt |= T_RCEXECARGS;
			}
		}
		else
			execargs = argv + optind;
	}
	#endif /* not NOFORK */
	return;
}

/***********************
  Get a home directory
************************/

char *get_homedir(char *user)
{
	struct passwd *pwd;
	pwd = getpwnam(user);

	if (pwd != 0)
		return (char *)pwd->pw_dir;
	else
		return (char *)getenv("HOME");
}

/**********************************
  Get your real user name (login)
***********************************/

char *anubis_getlogin(void)
{
	struct passwd *pwd;
	pwd = getpwuid(getuid());

	if (pwd != 0)
		return (char *)pwd->pw_name;
	else
		return (char *)getlogin();
}

/****************************************************
  Set USER's Group ID, User ID, and home directory.
*****************************************************/

#ifndef NOFORK
void anubis_changeowner(char *user)
{
	struct passwd *pwd;
	pwd = getpwnam(user);

	if (setgid(pwd->pw_gid) == 0)
		info(VERBOSE, "GID changed to %s's group id.", user);
	if (setuid(pwd->pw_uid) == 0)
		info(VERBOSE, "UID changed to %s's user id.", user);
	safe_strcpy(local_user, anubis_getlogin());
	if (chdir(pwd->pw_dir) == 0)
		info(VERBOSE, "CWD changed to %s's home directory.", user);
	return;
}
#endif /* not NOFORK */

/***************************
  Check a file permissions
****************************/

int check_file_mode(char *fpath)
{
	struct stat st;

	if (stat(fpath, &st) == -1)
		return 0;
	if ((st.st_mode & S_IRWXG) || (st.st_mode & S_IRWXO)) {
		anubis_error("Wrong permissions on %s", fpath);
		popt &= ~P_ERROR;
		return -1;
	}
	return 0;
}

/* EOF */

