/*
    errs.c

    This file is a part of Anubis.
    Copyright (C) 2001, 2002 The Anubis Team.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "headers.h"
#include "extern.h"

void anubis_error(char *format, ...)
{
	va_list arglist;
	char txt[512];

	va_start(arglist, format);
	#ifdef HAVE_VSNPRINTF
	vsnprintf(txt, 512, format, arglist);
	#else
	vsprintf(txt, format, arglist);
	#endif /* HAVE_VSNPRINTF */
	va_end(arglist);

	popt |= P_ERROR; /* for daemon mode */

	if (output != SILENT) {
		#ifndef NOLOGS
		if ((popt & P_DAEMON) && !(popt & P_FOREGROUND))
			syslog(LOG_ERR | LOG_MAIL, txt);
		else
		#endif /* not NOLOGS */
			if ((popt & P_FOREGROUND))
				mprintf("%s>%s [%d] %s", ansi_red, ansi_normal, (int)getpid(), txt);
			else
				mprintf("%s>%s %s", ansi_red, ansi_normal, txt);
	}
	if (!(popt & P_DAEMON) && !(popt & P_FOREGROUND))
		quit(EXIT_FAILURE);
	return;
}

void socket_error(void)
{
	anubis_error("Couldn't write on socket ! %s", strerror(errno));
	return;
}

void socks_proxy_error(char *txt)
{
	anubis_error("SOCKS Proxy: %s", txt);
	return;
}

/* EOF */

