/*
    headers.h

    This file is a part of Anubis.
    Copyright (C) 2001, 2002 The Anubis Team.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <ctype.h>
#include <unistd.h>
#include <stdarg.h>
#include <time.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/file.h>
#include <sys/fcntl.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <memory.h>
#include <sys/wait.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/resource.h>
#include <netdb.h>
#include <netinet/in.h>
#include <pwd.h>

#include "mem.h" /* xfree() */

#ifdef HAVE_CONFIG_H

 #include "config.h"
 #if !defined(HAVE_LIBSSL) || !defined(HAVE_LIBCRYPTO)
  || !defined(HAVE_OPENSSL_SSL_H)
  #define NOSSL
 #endif /* not HAVE_LIBSSL or not HAVE_LIBCRYPTO */
 #if !defined(HAVE_LIBGPGME) || !defined(HAVE_GPGME_H)
  #define NOGPG
 #endif /* not HAVE_LIBGPGME or not HAVE_GPGME_H */
 #if defined(HAVE_LIBPCRE) && defined(HAVE_PCRE_H)
  #define PCRE
 #endif /* HAVE_LIBPCRE and HAVE_PCRE_H */
 #ifndef HAVE_ARPA_INET_H
  #define NOARPAINETH
 #endif /* not HAVE_ARPA_INET_H */
 #if !defined(HAVE_SYSLOG) || !defined(HAVE_SYSLOG_H)
  #define NOLOGS
 #endif /* not HAVE_SYSLOG or not HAVE_SYSLOG_H */
 #if !defined(HAVE_REGEX_H)
  #define NOREGEX
 #endif /* not HAVE_REGEX_H */
 #ifndef HAVE_GETPASS
  #define NOGETPASS
 #endif /* not HAVE_GETPASS */
 #ifndef HAVE_SETPRIORITY
  #define NOPRI
 #endif /* not HAVE_SETPRIORITY */
 #ifndef HAVE_SETRLIMIT
  #define NOSETRLIMIT
 #endif /* not HAVE_SETRLIMIT */
 #ifndef HAVE_SOCKETPAIR
  #define NOSOCKETPAIR
 #endif /* not HAVE_SOCKETPAIR */
 #ifndef HAVE_GETHOSTNAME
  #define NOGETHOSTNAME
 #endif /* not HAVE_GETHOSTNAME */
 #ifndef HAVE_STRDUP
  #define NOSTRDUP
 #endif /* not HAVE_STRDUP */

#else /* hardcoded */

 #ifdef AMIGA
  #define NOGPG
  #define NOSETRLIMIT
 #endif /* AMIGA */
 #ifdef __BEOS__
  #define NOGETPASS
  #define NOPRI
  #define NOLOGS
  #define NOGPG
  #define NOSOCKETPAIR
  #define NOSETRLIMIT
 #endif /* __BEOS__ */

#endif /* HAVE_CONFIG_H */

/*
   OS detection.
*/

#ifdef AMIGA
 #define NOFORK /* hardcoded, don't touch it. */
#endif /* AMIGA */

/*
   End of OS detection.
*/

#ifdef NOPGP
 #ifndef NOGPG
  #define NOGPG
 #endif /* not GPG */
#endif /* not PGP */

#ifdef DEBUG_CODE
 #define debug_response(txt) \
 fprintf(stderr, "SERVER: %s", txt);
 #ifndef NOANSI
  #define NOANSI
 #endif /* not NOANSI */
#endif /* DEBUG_CODE */

#ifdef TUNNEL_OPTIMIZE
 #ifndef NOANSI
  #define NOANSI
 #endif /* not NOANSI */
#endif /* TUNNEL_OPTIMIZE */

#ifndef NOARPAINETH
 #include <arpa/inet.h>
#endif /* NOARPAINETH */
#ifdef NOSSL
 #include "md5.h"
#endif /* NOSSL */
#ifndef NOLOGS
 #include <syslog.h>
#endif /* not NOLOGS */
#ifdef PCRE
 #include <pcre.h>
 #ifndef NOREGEX
  #define NOREGEX
 #endif /* not NOREGEX */
#endif /* PCRE */
#ifndef NOREGEX
 #include <regex.h>
#endif /* not NOREGEX */
#ifndef NOSSL
 #include <openssl/crypto.h>
 #include <openssl/x509.h>
 #include <openssl/pem.h>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/rand.h>
#endif /* not NOSSL */

#ifndef __socklen_t_defined
 typedef int socklen_t;
 #define __socklen_t_defined
#endif /* not __socklen_t_defined */

#ifndef MAXPATHLEN
 #ifdef PATH_MAX
  #define MAXPATHLEN PATH_MAX
 #else
  #define MAXPATHLEN 1024
 #endif /* PATH_MAX */
#endif /* not MAXPATHLEN */
#ifndef NAME_MAX
 #define NAME_MAX 255
#endif /* not NAME_MAX */

#define LINEBUFFER 512
#define DEFAULT_HOSTNAME "localhost"
#define DEFAULT_GLOBAL_CONFIG_FILE "/etc/anubisrc"
#define DEFAULT_LOCAL_CONFIG_FILE ".anubisrc"
#define DEFAULT_SSL_PEM "anubis.pem"
#define DEFAULT_MIXMASTER_PATH "Mix/mix"
#define DEFAULT_SIGNATURE_FILE ".signature"
#define BEGIN_SMTP "---BEGIN SMTP---"
#define BEGIN_TUNNEL "---BEGIN TUNNEL---"
#define BEGIN_REGEX "---BEGIN REGEX---"
#define END_CONFIG "---END---"
#define CRLF "\r\n"

/* REGEX action methods */
#define COMMAND	0
#define HEADER	1

/* Tunnel methods */
#define CLIENT	0
#define SERVER	1

/* config file */
#define LOCAL	0
#define REMOTE	1

/* output modes */
#define DEBUG		3
#define VERBOSE		2
#define NORMAL		1
#define SILENT		0

#define OK "OK"
#define FAILED "FAILED"
#define WARNING "> WARNING:"

/* DO NOT EDIT ! */
#ifndef NOANSI
 #define ESC "\x1B"
 #define ANSI_NORMAL ESC"[0m"
 #define ANSI_GREEN  ESC"[1;32m"
 #define ANSI_RED    ESC"[1;31m"
 #define ANSI_BLUE   ESC"[1;36m"
 #define ANSI_YELLOW ESC"[1;33m"
 #define ANSI_OK "[  "ANSI_GREEN""OK""ANSI_NORMAL"  ]"
 #define ANSI_FAILED "["ANSI_RED""FAILED""ANSI_NORMAL"]"
 #define ANSI_WARNING ANSI_RED""WARNING""ANSI_NORMAL
#else
 #define ANSI_NORMAL ""
 #define ANSI_GREEN  ""
 #define ANSI_RED    ""
 #define ANSI_BLUE   ""
 #define ANSI_YELLOW ""
 #define ANSI_OK "[  "OK"  ]"
 #define ANSI_FAILED "["FAILED"]"
 #define ANSI_WARNING WARNING
#endif /* not NOANSI */

/* bit values for mopt */
#define M_FROM				0x00000001
#define M_REPLYTO			0x00000002
#define M_TO				0x00000004
#define M_SUBJECT			0x00000008
#define M_BODY				0x00000010
#define M_CC				0x00000020
#define M_BCC				0x00000040
#define M_EH1				0x00000080
#define M_EH2				0x00000100
#define M_SIGNATURE			0x00000200
#define M_ROT13				0x00000400
#define M_ROT13S			0x00000800
#define M_ROT13B			0x00001000
#define M_PGP				0x00002000
#define M_PGPSIGN			0x00004000
#define M_CHARSET			0x00008000
#define M_NOROOT			0x00010000
#define M_FILEATTACH		0x00020000
#define M_BODYATTACH		0x00040000
#define M_RM				0x00080000
#define M_RMRRT				0x00100000
#define M_RMPOST			0x00200000
#define M_RMHEADER			0x00400000
#define M_RMLT				0x00800000
#define M_RMRLT				0x01000000
#define M_RMPGP				0x02000000
#define M_RMPGPFILE			0x04000000
#define M_MIXMASTER			0x08000000
#define M_MID_CUSTOM		0x10000000
#define M_DATE_CUSTOM		0x20000000
#define M_XMAIL_CUSTOM		0x40000000
#define M_XMAIL_NO			0x80000000

/* bit values for popt */
#define P_SMTP				0x00000001
#define P_ERROR				0x00000002
#define P_SMTPAUTH			0x00000004
#define P_WG1				0x00000008
#define P_WG2				0x00000010
#define P_SOCKS				0x00000020
#define P_SOCKSV4			0x00000040
#define P_SOCKSUSERNAME		0x00000080
#define P_SOCKSPASSWORD		0x00000100
#define P_FAKENO			0x00000200
#define P_FAKEIP1			0x00000400
#define P_FAKEIP2			0x00000800
#define P_FAKEHOST			0x00001000
#define P_OVERLOAD			0x00002000
#define P_NOANSI			0x00004000
#define P_FOREGROUND_INIT	0x00008000
#define P_FOREGROUND		0x00010000
#define P_DAEMON			0x00020000
#define P_SSL_NO			0x00040000
#define P_SSL_ONLY			0x00080000
#define P_SSL_CLIENT		0x00100000
#define P_SSL_SERVER		0x00200000
#define P_NORC				0x00400000
#define P_ALTRC				0x00800000
#define P_REALUSER			0x01000000
#define P_USERNAME			0x02000000
#define P_PASSWORD			0x04000000

/* bit values for topt */
#define T_TUNNEL			0x00000001
#define T_NAMES				0x00000002
#define T_LOCAL_MTA			0x00000004
#define T_LOCAL_CONNECTION	0x00000008
#define T_AUTH_LOCAL		0x00000010
#define T_IDENT_ONLY		0x00000020
#define T_SETREMOTE_USER	0x00000040
#define T_STARTTLS			0x00000080
#define T_BOUNDARY			0x00000100
#define T_RCEXECARGS		0x00000200
#define T_STD_INOUT			0x00000400

#define safe_strcpy(s, ct) \
	(s[sizeof(s) - 1] = '\0', strncpy((char *)s, (char *)ct, sizeof(s) - 1))

/* mem.c */
void *xmalloc(int);
void *xrealloc(void *, int);
char *allocbuf(char *, int);
#ifdef NOSTRDUP
char *strdup(const char *);
#endif /* NOSTRDUP */

/* env.c */
void get_options(int, char *[]);
char *get_homedir(char *);
char *anubis_getlogin(void);
#ifndef NOFORK
void anubis_changeowner(char *);
#endif /* not NOFORK */
int  check_file_mode(char *);

/* errs.c */
void anubis_error(char *, ...);
void socket_error(void);
void socks_proxy_error(char *);

/* log.c */
void mprintf(char *format, ...);
void info(int, char *, ...);

/* net.c */
int make_remote_connection(char *, unsigned int);
int make_local_connection(char *, char **);
int bind_and_listen(char *, unsigned int);
int swrite_char(int, void *, char);
void swrite(int, void *, char *);
void sendline(void *, char *);
int recvline(int, void *, void *, int);
void get_response_smtp(int, void *, char *, int);
void close_socket(int);

/* proxy.c */
void check_all_proxies(char *, unsigned int *);
int check_wingates(int, char *, unsigned int);
int check_socks_proxy(int, char *, unsigned int);

/* tunnel.c */
void smtp_tunnel_init(char *, unsigned int);
#if !defined(NOREGEX) || defined(PCRE)
int regex_match(char *, char *);
#endif /* not NOREGEX || PCRE */

/* smtp.c */
#ifndef TUNNEL_OPTIMIZE
void smtp_client_init(void);
#endif /* not TUNNEL_OPTIMIZE */

/* misc.c */
void set_up_variables(void);
void parse_hostname_port(char *, char *, unsigned int *);
void parse_hostname_host(char *, char *, unsigned int *);
char *parse_address(char *);
void remline(char *, char *);
void remcrlf(char *);
void getoneline(char *, char *, int);
void change_to_lower(char *);
char *get_time_string(int);
void check_rot13(void);
void cram_md5(char *, char *, unsigned char *);
#ifndef NOFORK
 char **gen_execargs(const char *);
 void free_execargs(char **);
#endif /* not NOFORK */

/* mime.c */
void check_all_files(char *);
void close_all_files(void);
#ifndef TUNNEL_OPTIMIZE
 void encode_mime(int, void *);
 char *base64encode_string(char *, int);
 int  base64decode_string(char *, char **);
 void attach_pgp_file(int, void *);
 char *examine_file_type(char *);
#endif /* not TUNNEL_OPTIMIZE */

/* rand.c */
int rand_se(int);
int classc(void);
char *rand_md5(void);
char *generate_mid(char *);

/* interactive.c */
#ifndef TUNNEL_OPTIMIZE
void interactive_mode(void);
#endif /* not TUNNEL_OPTIMIZE */
void auth_username(char *, int);
void auth_password(char *, char *, int);

/* rc.c */
void open_config_file(int, char *);
void read_config_file(void);
char *parse_line_option(char *);
#if !defined(NOREGEX) || defined(PCRE)
 int read_regex_block(int, char *, int);
 int read_action_block(char *);
#endif /* not NOREGEX || PCRE */
unsigned long get_position(char *);

/* help.c */
void print_version(void);
void print_usage(void);

/* quit.c */
void sig_exit(int);
void sig_timeout(int);
void free_mem(void);
void quit(int);

/* ssl.c */
#ifndef NOSSL
 void init_ssl_libs(void);
 SSL_CTX *init_ssl_client(void);
 SSL_CTX *init_ssl_server(void);
 SSL *start_ssl_client(int, SSL_CTX *);
 SSL *start_ssl_server(int, SSL_CTX *);
 void end_ssl(int, SSL *, SSL_CTX *);
#endif /* not NOSSL */

/* pgp.c */
#ifndef NOGPG
void check_pgp(void);
#endif /* not NOGPG */

/* EOF */

