/*
    help.c

    This file is a part of Anubis.
    Copyright (C) 2001, 2002 The Anubis Team.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "headers.h"
#include "extern.h"

void print_version(void)
{
	char *env = getenv("MACHTYPE");

	printf("\n%s\n", version);
	printf("%s\n", copyright);
	printf("\nBuilt-in support for: ");
	#ifdef DEBUG_CODE
	 printf("DEBUG ");
	#endif /* DEBUG_CODE */
	#ifndef NOREGEX
	 printf("REGEX ");
	#else
	 #ifdef PCRE
	  printf("PCRE ");
	 #endif /* PCRE */
	#endif /* not NOREGEX */
	#ifndef NOSSL
	 printf("SSL ");
	#endif /* not NOSSL */
	#ifndef NOGPG
	 printf("GPG ");
	#endif /* not NOGPG */
	#ifndef TUNNEL_OPTIMIZE
	 printf("CLIENT ");
	#endif /* not TUNNEL_OPTIMIZE */

	printf("\nRunning on: %s\n", env ? env : "unknown");
	system("uname -a");
	printf("\nThis program is free software; you can redistribute it and/or modify\n");
	printf("it under the terms of the GNU General Public License as published by\n");
	printf("the Free Software Foundation; either version 2 of the License, or\n");
	printf("(at your option) any later version.\n");
	printf("\nThis program is distributed in the hope that it will be useful,\n");
	printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
	printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
	printf("GNU General Public License for more details.\n");
	printf("\nYou should have received a copy of the GNU General Public License\n");
	printf("along with this program; if not, write to the Free Software\n");
	printf("Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\n");
	exit(0);
}

void print_usage(void)
{
	printf("\nUsage: anubis [options]\n");
	printf(
	"\n SMTP Tunnel\t"
	"[-t arg] "
	"[-l arg] "
	"[-h arg] "
	"[-f] "
	"[options]"
	);
	#ifndef TUNNEL_OPTIMIZE
	printf(
	"\n SMTP client\t"
	"-h arg "
	"-F arg "
	"-T arg "
	"-S arg "
	"-B arg "
	"[options] "
	);
	#endif /* not TUNNEL_OPTIMIZE */
	printf("\n");
	printf("\nAn outgoing mail processor (The SMTP Tunnel) options:\n\n");
	printf(" -t [HOST:]PORT        SMTP tunneling mode {default host is INADDR_ANY}\n");
	#ifndef NOFORK
	printf(" -l, --local-mta FILE  Execute a local inetd-type program (instead of -h).\n");
	#endif /* not NOFORK */
	printf(" -f, --foreground      foreground mode (don't fork)\n");
	printf(" -i, --sio             Use the Tunnel as described in RFC 821 on standard\n");
	printf("                       input and output.\n");
	if (getuid() == 0 && getgid() == 0)
	printf("     --setremoteuser   Set a remote user's GroupID, UserID, and home directory.\n");
	#ifndef TUNNEL_OPTIMIZE
	printf("\nGeneric options:\n\n");
	printf(" -h HOST[:PORT]        Specify SMTP host name or IP address.\n");
	printf(" -F, --from ADDR       Specify a sender address.\n");
	printf(" -r, --realuser        Your real user name and host name in the sender address.\n");
	printf(" -R, --reply-to ADDR   Reply-To:\n");
	printf(" -T, --to ADDR         Specify a recipient.\n");
	printf("     --cc ADDR         Specify the carbon-copy (CC) recipients.\n");
	printf("     --bcc ADDR        Specify the blind-carbon-copy (BCC) recipients.\n");
	printf(" -S, --subject ARG     Specify the subject of the message.\n");
	printf(" -B, --body ARG        Message body.\n");
	printf("     --body-file FILE  Include a text file into the body of a message.\n");
	printf("     --signature       Attach the .signature file to your message.\n");
	#endif /* not TUNNEL_OPTIMIZE */
	printf("\nSOCKS Proxy options:\n\n");
	printf(" --socks-proxy HOST[:PORT]  SOCKS Proxy host name or IP address.\n");
	printf(" --socks-v4                 Use the SOCKSv4 protocol {SOCKSv5 is a default}.\n");
	printf(" --socks-username USERNAME  Specify a SOCKS Proxy user name.\n");
	printf(" --socks-password PASSWORD  Specify a SOCKS Proxy password.\n");
	printf("\nWinGate(s) options:\n\n");
	printf(" --wingate1 HOST[:PORT]  WinGate 1 host name or IP address {none}.\n");
	printf(" --wingate2 HOST[:PORT]  WinGate 2 host name or IP address {none}.\n");
	#ifndef TUNNEL_OPTIMIZE
	printf("\nRemailer Type-I options (\"To:\" address is a remailer):\n\n");
	printf(" --rm-rrt ADDR    Request Remailing To: (Anon-To:)\n");
	printf(" --rm-post GROUP  News post via remailer (Anon-Post-To:).\n");
	#ifndef NOGPG
	printf(" --rm-pgp KEY     Encrypt a message with a remailer's GPG/PGP public key.\n");
	#endif /* not NOGPG */
	printf(" --rm-pgp-file    Attach the file already encrypted with a remailer's PGP key.\n");
	printf(" --rm-header ARG  An extra header in remailed message.\n");
	printf(" --rm-lt XX:XX    Specify a Latent-Time: (e.g. --rm-lt \"2:00\" = 2h)\n");
	printf(" --rm-rlt         A random Latent-Time over the Latent-Time: (a random delay)\n");
	printf("\nRemailer Type-II option:\n\n");
	printf(" --mix            Redirect all to the Mixmaster's input.\n");
	printf("                  default Mixmaster path is: $HOME/Mix/mix\n");
	#ifndef NOGPG
	printf("\nPGP options:\n\n");
	printf(" --pgp-sign         Sign a message.\n");
	printf(" --pgp-encrypt KEY  Encrypt a message (comma separated list of keys).\n");
	#endif /* not NOGPG */
	#endif /* not TUNNEL_OPTIMIZE */
	printf("\nAdvanced options:\n\n");
	#ifndef NOSSL
	printf(" --nossl          Turn off the ESMTP TLS/SSL support.\n");
	printf(" --ssl-only       Exit if TLS/SSL is unavailable at a remote host.\n");
	printf(" --cert FILE      Certificate file to use, PEM format assumed\n");
	printf("                  (default is %s).\n", DEFAULT_SSL_PEM);
	printf(" --key FILE       Private Key file to use, PEM format assumed, in cert file if\n");
	printf("                  not specified (default is %s).\n", DEFAULT_SSL_PEM);
	#endif /* not NOSSL */
	#ifndef TUNNEL_OPTIMIZE
	printf(" --noroot         MAIL FROM: <someone@somewhere>\n");
	printf("                  instead of <root@smtp.server>\n");
	printf(" --smtp-auth      Turn on the ESMTP Authentication.\n");
	printf(" --smtp-mode ARG  mode {MAIL} (MAIL,SEND,SOML,SAML)\n");
	printf(" --rot13          ROT-13 encoding (subject & message body)\n");
	printf(" --rot13-subject  ROT-13 encoding (subject)\n");
	printf(" --rot13-body     ROT-13 encoding (message body)\n");
	printf(" --mid ARG        Specify a custom Message-ID, e.g. \"blabla@hostname\"\n");
	printf(" --xmailer ARG    Specify a custom X-Mailer, e.g. \"Anubis :)\"\n");
	printf(" --xmailer-no     Do not use X-Mailer.\n");
	printf(" --charset ARG    Specify a custom charset, e.g. iso-8859-2 {def: US-ASCII}\n");
	printf(" --fake-no        Do not use the \"Stealth\" mode.\n");
	printf(" --fake-ip1 ARG   e.g. 127.0.0.1 {default: a random IP address}\n");
	printf(" --fake-ip2 ARG   e.g. 127.0.0.1 {default: a random IP address}\n");
	printf(" --fake-host ARG  Fake host name for the \"Stealth\" mode {default: unknown}\n");
	printf(" --file FILE      Attach a file to your message using MIME (Base64 encoding)\n");
	printf(" --uuencode       Uuencode instead of Base64 encoding.\n");
	printf(" --extra-h1 ARG   An extra header - e.g. \"Sender: J.Smith@abc.com\"\n");
	printf(" --extra-h2 ARG   A second extra header.\n");
	printf(" --overload-helo  Overload HELO command -> Old Sendmail HELO bug\n");
	#endif /* not TUNNEL_OPTIMIZE */
	printf("\nOutput options:\n\n");
	#ifndef NOANSI
	printf("     --noansi     Use \"black & white\" output mode.\n");
	#endif /* not NOANSI */
	printf(" -s, --silent     Work silently.\n");
	printf(" -v, --verbose    Work noisily.\n");
	#ifdef DEBUG_CODE
	printf(" -D, --debug      Debug mode.\n");
	#endif /* DEBUG_CODE */
	printf("\nMisc. options:\n\n");
	printf(" --altrc FILE     Specify alternate run control file.\n");
	printf(" --norc           Ignore the run control file.\n");
	#ifndef NOPRI
	printf(" --pri ARG        Set the task priority <-20,+20>\n");
	#endif /* not NOPRI */
	printf(" --version        Print version number, copyright, and OS info.\n");
	printf(" --help           It's obvious...\n\n");
	printf("Report bugs to: <read the AUTHORS file>\n");
	exit(0);
}

/* EOF */

