/*
    interactive.c

    This file is a part of Anubis.
    Copyright (C) 2001, 2002 The Anubis Team.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "headers.h"
#include "extern.h"

/***************************
  Simple interactive mode
****************************/

#ifndef TUNNEL_OPTIMIZE
void interactive_mode(void)
{
	char line[LINEBUFFER];
	memset(line, 0, LINEBUFFER);

	if (!(mopt & M_FROM)) {
		printf("From: [someone@somewhere] : ");
		if (!fgets(line, LINEBUFFER, stdin)
		|| !isatty(fileno(stdin))) {
			if (output == SILENT)
				putchar('\n');
		}
		line[strlen(line) - 1] = '\0';
		if (strlen(line) != 0) {
			from = allocbuf(line, 256);
			if (from)
				mopt |= M_FROM;
		}
	}
	if (!(mopt & M_TO)) {
		printf("To:   [someone@somewhere] : ");
		if (!fgets(line, LINEBUFFER, stdin)
		|| !isatty(fileno(stdin))) {
			if (output == SILENT)
				putchar('\n');
		}
		line[strlen(line) - 1] = '\0';
		if (strlen(line) != 0) {
			to = allocbuf(line, 256);
			if (to)
				mopt |= M_TO;
		}
	}
	if (!(mopt & M_SUBJECT)) {
		printf("Subject: ");
		if (!fgets(line, LINEBUFFER, stdin)
		|| !isatty(fileno(stdin))) {
			if (output == SILENT)
				putchar('\n');
		}
		line[strlen(line) - 1] = '\0';
		subject = allocbuf(line, 512);
	}

	/***************************
	   Multiline data input
	****************************/
	if (!(mopt & M_BODY)) {
		char *env_tmp = 0;
		int columns;
		int ok = 1;

		env_tmp = getenv("COLUMNS");
		if (env_tmp) {
			columns = atoi(env_tmp);
			if (columns >= 80)
				ok = 1;
			else
				ok = 0;
		}
		if (isatty(fileno(stdin))) {
			if (ok == 1) {
				printf("Message Body [end with \".\" on a line by itself]"
				"                        76->*<-76\n");
			}
			else
				printf("Message Body [end with \".\" on a line by itself]\n");
		}
		msgbody = (char *)xmalloc(1);

		for (;;)
		{
			if (!fgets(line, LINEBUFFER, stdin))
				break;
			line[strlen(line) - 1] = '\0';
			if (strncmp(line, ".\0", 2) == 0)
				break;

			msgbody = (char *)xrealloc((char *)msgbody,
			strlen(msgbody) + strlen(line) + 4);
			strncat(msgbody, line, strlen(line));
			strncat(msgbody, CRLF, 2);
		}
	}
	if (!(mopt & M_FROM))
		anubis_error("'From' field is missing.");
	if (!(mopt & M_TO))
		anubis_error("'To' field is missing.");
	return;
}
#endif /* not TUNNEL_OPTIMIZE */

void auth_username(char *user, int size)
{
	memset(user, 0, size);
	printf("Username: ");
	if (!fgets(user, size, stdin) || !isatty(fileno(stdin))) {
		if (output != SILENT)
			putchar('\n');
	}
	user[strlen(user) - 1] = 0;
	if (strlen(user) == 0) {
		anubis_error("No user name specified.");
	}
	return;
}

void auth_password(char *txt, char *pass, int size)
{
	#ifdef NOGETPASS
	memset(pass, 0, size);
	printf("%s", txt);
	if (!fgets(pass, size, stdin) || !isatty(fileno(stdin))) {
		if (output != SILENT)
			putchar('\n');
	}
	pass[strlen(pass) - 1] = 0;
	#else
	char *ptr = getpass(txt);
	memset(pass, 0, size);
	strncpy(pass, ptr, size);
	memset(ptr, 0, strlen(ptr));
	#endif /* NOGETPASS */
	return;
}

/* EOF */

