/**************************************************************************
 Open your mind. Open (your) Source. Human knowledge belongs to the world.
***************************************************************************/
/*
  Anubis v2.0.0 BETA-5 - an outgoing mail processor, the SMTP tunnel,
                         and secure SMTP client.

  Copyright (C) 2001, 2002 The Anubis Team.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  This product includes software developed by the University of California,
  Berkeley and its contributors.

  ************************************
  | 6258d3912c5a6e2a9a90f15cfeada865 |
  ************************************
*/

#include "headers.h"
#include "main.h"

int main(int argc, char *argv[])
{
	/*
	   Initialize the Anubis.
	*/

	#if !defined(DEBUG_CODE) && !defined(NOSETRLIMIT)
	{
		struct rlimit corelimit;
		corelimit.rlim_cur = 0;
		corelimit.rlim_max = 0;
		setrlimit(RLIMIT_CORE, &corelimit);
	}
	#endif /* not DEBUG_CODE and not NOSETRLIMIT */

	/*
	   Signal handling...
	*/

	signal(SIGILL,  sig_exit);
	signal(SIGINT,  sig_exit);
	signal(SIGTERM, sig_exit);
	signal(SIGHUP,  sig_exit);
	signal(SIGQUIT, sig_exit);
	signal(SIGPIPE, SIG_IGN);
	signal(SIGALRM, sig_timeout);

	#ifdef TUNNEL_OPTIMIZE
	topt |= T_TUNNEL;
	#else
	popt |= P_SMTP;
	#endif /* TUNNEL_OPTIMIZE */

	/*
	   Disable ANSI color output if needed.
	*/

	#ifndef NOANSI
	if (!isatty(fileno(stdout)) || (popt & P_NOANSI)
	|| ((topt & T_TUNNEL) && !(popt & P_FOREGROUND_INIT))) {
		strncpy(ansi_normal, "", 1);
		strncpy(ansi_red, "", 1);
		strncpy(ansi_green, "", 1);
		strncpy(ansi_blue, "", 1);
		strncpy(ansi_yellow, "", 1);
		strncpy(ansi_ok, OK, sizeof(ansi_ok));
		strncpy(ansi_failed, FAILED, sizeof(ansi_failed));
		strncpy(ansi_warning, WARNING, sizeof(ansi_warning));
	}
	#endif /* not NOANSI */

	/*
	   Process the command-line options...
	*/

	get_options(argc, argv);
	safe_strcpy(local_user, anubis_getlogin());

	/*
	   Read the run control file.
	*/

	if (!(popt & P_NORC)) {
		open_config_file(LOCAL, local_user);
		read_config_file();
		if (fp_config)
			fp_config = (FILE *)fclose(fp_config);
	}
	set_up_variables();

	/*
	   Print the version banner.
	*/

	if (!(topt & T_TUNNEL))
		mprintf("%s, %s\n", version, copyright);

	/*
	   Initialize the OpenSSL libraries and PRNG.
	*/

	#ifndef NOSSL
	if (!(popt & P_SSL_NO))
		init_ssl_libs();
	#endif /* not NOSSL */

	/*
	   Run the Anubis SMTP Tunnel/outgoing mail processor.
	*/

	if (topt & T_TUNNEL) {
		smtp_tunnel_init(tunnel_hostname, tunnel_portnumber);
		quit(0);
	}

	/*
	   Run the Anubis SMTP client (default).
	*/

	#ifndef TUNNEL_OPTIMIZE
	interactive_mode();
	smtp_client_init();
	#endif /* not TUNNEL_OPTIMIZE */

	return 0;
}

/* EOF */

