/*
    mem.c

    This file is a part of Anubis.
    Copyright (C) 2001, 2002 The Anubis Team.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "headers.h"

void *xmalloc(int n)
{
	void *p;

	p = (void *)malloc(n);
	memset(p, 0, n);

	if (p == (void *)0) {
		anubis_error("malloc() failed. Cannot allocate enough memory.");
		quit(EXIT_FAILURE); /* force exit */
	}
	return p;
}

void *xrealloc(void *p, int n)
{
	if (p == 0)
		return xmalloc(n);

	p = (void *)realloc(p, n);
	if (p == (void *)0) {
		anubis_error("realloc() failed.");
		quit(EXIT_FAILURE); /* force exit */
	}
	return p;
}

char *allocbuf(char *s, int maxsize)
{
	char *p = 0;
	int len = strlen(s);

	if (maxsize != 0) {
		if (len > maxsize)
			len = maxsize;
	}
	len++;

	p = (char *)xmalloc(len);
	if (p) {
		strncpy(p, s, len - 1);
		return p;
	}
	else
		return 0;
}

#ifdef NOSTRDUP
char *strdup(const char *s)
{
	char *p = 0;
	int len = strlen(s);

	p = (char *)malloc(len + 1);
	strncpy(p, s, len);
	return p;
}
#endif /* NOSTRDUP */

/* EOF */

