/*
    mime.c

    This file is a part of Anubis.
    Copyright (C) 2001, 2002 The Anubis Team.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "headers.h"
#include "extern.h"

static void attach_text_file(void);
static void attach_signature_file(char *);
#ifndef TUNNEL_OPTIMIZE
static void open_file(void);
static void open_pgp_file(void);

struct mime_struct
{
	int id;
	char *type;
} static mime[] =
{
	{0,	"x-unknown"},
	{1,	"text/plain"},					/* asc txt */
	{2,	"text/html"},					/* htm html shtml */
	{3,	"text/sgml"},					/* sgml sgm */
	{4,	"application/octet-stream"},	/* bin exe lha lzh class */
	{5,	"application/mac-binhex40"},	/* hqx */
	{6,	"application/postscript"},		/* ps */
	{7,	"application/rtf"},				/* rtf */
	{8,	"application/pdf"},				/* pdf */
	{9,	"application/x-tar"},			/* tar */
	{10,	"application/x-gzip"},			/* gz tgz */
	{11,	"application/x-bzip2"},			/* bz2 */
	{12,	"application/x-texinfo"},		/* texi */
	{13,	"application/zip"},				/* zip */
	{14,	"image/gif"},					/* gif */
	{15,	"image/jpeg"},					/* jpeg jpg jpe */
	{16,	"image/png"},					/* png */
	{17,	"image/tiff"},					/* tiff tif */
	{18,	"video/mpeg"},					/* mpeg mpg mpe divx */
	{19,	"video/x-msvideo"},				/* avi */
	{20,	"video/quicktime"},				/* qt mov */
	{21,	"video/x-sgi-movie"},			/* movie */
	{22,	"audio/basic"},					/* au snd */
	{23,	"audio/mpeg"},					/* mpga mp2 mp3 */
	{24,	"audio/x-pn-realaudio"},		/* ram rm */
	{25,	"audio/x-realaudio"}			/* ra */
};
struct fsuffix_struct
{
	int id;
	char *suffix;
} static fsuffix[] =
{
	{1,	"txt"	}, {1,	"asc"	}, {1,	"c"		},
	{1,	"cc"	}, {1,	"cpp"	}, {1,	"h"		},
	{1,	"asm"	}, {1,	"s"		}, {1,	"pl"	},
	{1,	"md5"	}, {2,	"htm"	}, {2,	"html"	},
	{2,	"shtml"	}, {3,	"sgml"	}, {3,	"sgm"	},
	{4,	"o"		}, {4,	"exe"	}, {4,	"elf"	},
	{4,	"bin"	}, {4,	"lib"	}, {4,	"dll"	},
	{4,	"doc"	}, {4,	"pkg"	}, {4,	"z"		},
	{4,	"lha"	}, {4,	"lzh"	}, {4,	"lzx"	},
	{4,	"pgp"	}, {4,	"gpg"	}, {4,	"sig"	},
	{4,	"key"	}, {4,	"dvi"	}, {4,	"so"	},
	{4,	"class"	}, {5,	"hqx"	}, {6,	"ps"	},
	{6,	"eps"	}, {7,	"rtf"	}, {8,	"pdf"	},
	{9,	"tar"	}, {10,	"gz"	}, {10,	"tgz"	},
	{11,	"bz2"	}, {12,	"texi"	}, {13,	"zip"	},
	{14,	"gif"	}, {15,	"jpg"	}, {15,	"jpeg"	},
	{15,	"jpe"	}, {16,	"png"	}, {17,	"tif"	},
	{17,	"tiff"	}, {18,	"mpg"	}, {18,	"mpeg"	},
	{18,	"mpe"	}, {18,	"divx"	}, {19,	"avi"	},
	{20,	"qt"	}, {20,	"mov"	}, {21,	"movie"	},
	{22,	"au"	}, {22,	"snd"	}, {23,	"mpga"	},
	{23,	"mp2"	}, {23,	"mp3"	}, {24,	"ram"	},
	{24,	"rm"	}, {25,	"ra"	}
};
#define NKEYS_S (sizeof(fsuffix) / sizeof(struct fsuffix_struct))

const char *trans_ptr;
const char uu_std[64] =
{
  '`', '!', '"', '#', '$', '%', '&', '\'',
  '(', ')', '*', '+', ',', '-', '.', '/',
  '0', '1', '2', '3', '4', '5', '6', '7',
  '8', '9', ':', ';', '<', '=', '>', '?',
  '@', 'A', 'B', 'C', 'D', 'E', 'F', 'G',
  'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
  'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W',
  'X', 'Y', 'Z', '[', '\\', ']', '^', '_'
};
const char uu_base64[64] =
{
  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
  'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
  'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
  'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
  'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
  'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
  'w', 'x', 'y', 'z', '0', '1', '2', '3',
  '4', '5', '6', '7', '8', '9', '+', '/'
};
#define ENC(Char) (trans_ptr[(Char) & 077])

/***************************************
  Uuencode & Base64 encoding function
****************************************/

void encode_mime(int method, void *sd)
{
	register int ch, n;
	register char *p = 0;
	char buf[80];

	if (fp_global == 0)
		return;
	rewind(fp_global);

	while (1)
	{
		n = 0;
		do {
			register int m = fread(buf, 1, 45 - n, fp_global);

			if (m == 0)
				break;
			n += m;
		} while (n < 45);

		if (n == 0)
			break;

		if (trans_ptr == uu_std) {
			if (swrite_char(method, sd, ENC(n)) < 0)
				break;
		}
		for (p = buf; n > 2; n -= 3, p += 3)
		{
			ch = *p >> 2;
			ch = ENC(ch);
			if (swrite_char(method, sd, ch) < 0)
				break;

			ch = ((*p << 4) & 060) | ((p[1] >> 4) & 017);
			ch = ENC(ch);
			if (swrite_char(method, sd, ch) < 0)
				break;
			ch = ((p[1] << 2) & 074) | ((p[2] >> 6) & 03);
			ch = ENC(ch);
			if (swrite_char(method, sd, ch) < 0)
				break;
			ch = p[2] & 077;
			ch = ENC(ch);
			if (swrite_char(method, sd, ch) < 0)
				break;
		}
		if (n != 0)
			break;
		swrite(method, sd, CRLF);
	}

	while (n != 0)
	{
		char c1 = *p;
		char c2 = n == 1 ? 0 : p[1];

		ch = c1 >> 2;
		ch = ENC(ch);
		if (swrite_char(method, sd, ch) < 0)
			break;

		ch = ((c1 << 4) & 060) | ((c2 >> 4) & 017);
		ch = ENC(ch);
		if (swrite_char(method, sd, ch) < 0)
			break;

		if (n == 1)
			ch = trans_ptr == uu_std ? ENC('\0') : '=';
		else {
			ch = (c2 << 2) & 074;
			ch = ENC(ch);
		}
		if (swrite_char(method, sd, ch) < 0)
			break;
		ch = trans_ptr == uu_std ? ENC('\0') : '=';
		if (swrite_char(method, sd, ch) < 0)
			break;

		swrite(method, sd, CRLF);
		break;
	}

	if (ferror(fp_global))
		anubis_error("File I/O error - %s !", strerror(errno));
	if (trans_ptr == uu_std) {
		swrite_char(method, sd, ENC('\0'));
		swrite(method, sd, CRLF);
	}
	return;
}

/*****************************************************************
  Base64 encoding/decoding functions for strings from Exim 3.22.
  Modified by The Anubis Team for Anubis v2.
******************************************************************/

char *base64encode_string(char *clear, int len)
{
	char *code = xmalloc(4 * ((len + 2) / 3) + 1);
	char *p = code;

	while (len-- > 0)
	{
		int x, y;
		x = *clear++;
		*p++ = uu_base64[(x >> 2) & 63];

		if (len-- <= 0) {
			*p++ = uu_base64[(x << 4) & 63];
			*p++ = '=';
			*p++ = '=';
			break;
		}
		y = *clear++;
		*p++ = uu_base64[((x << 4) | ((y >> 4) & 15)) & 63];

		if (len-- <= 0) {
			*p++ = uu_base64[(y << 2) & 63];
			*p++ = '=';
			break;
		}
		x = *clear++;
		*p++ = uu_base64[((y << 2) | ((x >> 6) & 3)) & 63];
		*p++ = uu_base64[x & 63];
	}
	*p = 0;
	return code;
}

static unsigned char uu_base64_decode[] =
{
	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255, /*  0-15 */
	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255, /* 16-31 */
	255,255,255,255,255,255,255,255,255,255,255,62,255,255,255,63,   /* 32-47 */
	52,53,54,55,56,57,58,59,60,61,255,255,255,255,255,255,           /* 48-63 */
	255,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,                          /* 64-79 */
	15,16,17,18,19,20,21,22,23,24,25,255,255,255,255,255,            /* 80-95 */
	255,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,                /* 96-111 */
	41,42,43,44,45,46,47,48,49,50,51,255,255,255,255,255             /* 112-127*/
};

int base64decode_string(char *code, char **ptr)
{
	int x, y;
	char *result = xmalloc(3 * (strlen(code) / 4) + 1);
	*ptr = result;

	while ((x = (unsigned char)(*code++)) != 0)
	{
		if (x > 127 || (x = uu_base64_decode[x]) == 255)
			return -1;
		if ((y = (unsigned char)(*code++)) == 0 || (y = uu_base64_decode[y]) == 255)
			return -1;
		*result++ = (x << 2) | (y >> 4);

		if ((x = (unsigned char)(*code++)) == '=') {
			if (*code++ != '=' || *code != 0)
				return -1;
		}
		else {
			if (x > 127 || (x = uu_base64_decode[x]) == 255)
				return -1;
			*result++ = (y << 4) | (x >> 2);
			if ((y = (unsigned char)(*code++)) == '=') {
				if (*code != 0)
					return -1;
			}
			else {
				if (y > 127 || (y = uu_base64_decode[y]) == 255)
					return -1;
				*result++ = (x << 6) | y;
			}
		}
	}
	*result = 0;
	return result - *ptr;
}

#endif /* not TUNNEL_OPTIMIZE */

/**************
   Files I/O
***************/

void check_all_files(char *user)
{
	char cwd[1024];
	memset(cwd, 0, sizeof(cwd));

	if (topt & T_TUNNEL) {
		getcwd(cwd, sizeof(cwd));
		chdir(get_homedir(user));
	}

	#ifndef TUNNEL_OPTIMIZE
	if (mopt & M_FILEATTACH)
		open_file();
	else if (mopt & M_RMPGPFILE)
		open_pgp_file();
	#endif /* not TUNNEL_OPTIMIZE */

	if (mopt & M_BODYATTACH)
		attach_text_file();
	if (mopt & M_SIGNATURE)
		attach_signature_file(user);
	if (topt & T_TUNNEL)
		chdir(cwd);
	return;
}

#ifndef TUNNEL_OPTIMIZE
static void open_file(void)
{
	#define C_MSDOS	"\\"
	#define C_UNIX	"/"
	struct stat st;
	char c_style[2];

	if ((fp_global = fopen(path, "r")) == 0) {
		anubis_error("fopen() failed. - %s", strerror(errno));
		return;
	}
	if (trans_ptr == uu_std) { /* Uuencode */
		if (fstat(fileno(fp_global), &st) == 0)
			 uu_mode = st.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO);
	}
	if (strstr(path, C_MSDOS)) /* MS-DOS style */
		strncpy(c_style, C_MSDOS, sizeof(c_style));
	else if (strstr(path, C_UNIX)) /* UNIX style */
		strncpy(c_style, C_UNIX, sizeof(c_style));

	if (strstr(path, c_style)) {
		char *ptr1 = 0;
		char *ptr2 = 0;
		ptr1 = strtok(path, c_style);
		for (;;)
		{
			ptr1 = strtok(0, c_style);
			if (ptr1 == 0) {
				filename = (char *)xmalloc(strlen(ptr2) + 1);
				strncpy(filename, ptr2, strlen(ptr2));
				break;
			}
			ptr2 = ptr1;
		}
	}
	else {
		filename = (char *)xmalloc(strlen(path) + 1);
		strncpy(filename, path, strlen(path));
	}
	return;
}

static void open_pgp_file(void)
{
	unsigned long nbytes;
	int bytes = 256;
	char buf[256];

	if ((fp_global = fopen(rm_pgp_file, "r")) == 0) { /* GPG/PGP - a clear text */
		anubis_error("GnuPG/PGP encrypted file error. %s", strerror(errno));
		return;
	}

	fseek(fp_global, 0L, SEEK_END);
	clearerr(fp_global);
	nbytes = ftell(fp_global);
	rewind(fp_global);

	if (nbytes <= 256) {
		bytes = nbytes;
		fread(buf, 1, nbytes, fp_global);
	}
	else
		fread(buf, 1, bytes, fp_global);

	if (!(strstr(buf, "--BEGIN PGP"))) {
		anubis_error("The %s file is not a GnuPG/PGP encrypted file "
		"(a clear text).", rm_pgp_file);
	}
	rewind(fp_global);
	return;
}

void attach_pgp_file(int method, void *sd)
{
	#define BUFSIZE 4096
	char buf[BUFSIZE];
	unsigned long flength;
	int count;

	if (fp_global == 0)
		return;

	fseek(fp_global, 0L, SEEK_END);
	flength = ftell(fp_global);
	rewind(fp_global);

	while(flength > 0)
	{
		memset(buf, 0, sizeof(buf));
		if (flength < BUFSIZE)
			count = (int)flength;
		else
			count = BUFSIZE;

		fread(buf, 1, count, fp_global);
		swrite(method, sd, buf);
		flength -= count;
	}
	return;
}
#endif /* not TUNNEL_OPTIMIZE */

static void attach_text_file(void)
{
	int c;
	FILE *fptxt;
	char *buf = 0;
	unsigned long nbytes;
	unsigned long nlines = 0;

	if ((fptxt = fopen(body_file, "r")) == 0) {
		anubis_error("fopen() failed. %s", strerror(errno));
		return;
	}

	while ((c = getc(fptxt)) != EOF)
	{
		if (c == '\n')
			++nlines;
	}
	fseek(fptxt, 0L, SEEK_END);
	clearerr(fptxt);
	nbytes = ftell(fptxt);
	rewind(fptxt);

	buf = (char *)xmalloc(nbytes + nlines + 4);

	while ((c = getc(fptxt)) != EOF)
	{
		if (c == '\n') /* convert LF -> CRLF */
			strncat(buf, CRLF, 2);
		else if (c == '\r') /* ignore CR */
			c = 0; /* Noop... Do nothing. */
		else {
			char tmp[2];
			tmp[0] = (char)c;
			tmp[1] = '\0';
			strncat(buf, tmp, 1);
		}
	}
	msgbody = (char *)xrealloc((char *)msgbody,
	strlen(msgbody) + strlen(buf) + 1);
	strncat(msgbody, buf, strlen(buf));

	free(buf);
	if (fptxt)
		fptxt = (FILE *)fclose(fptxt);
	return;
}

static void attach_signature_file(char *user)
{
	FILE *fpsig;
	int n;
	unsigned long nbytes;
	unsigned long nlines = 0;
	char *buf = 0;
	char *home = 0;
	char *signature_path = 0;
	char *signature_file = (char *)DEFAULT_SIGNATURE_FILE;

	home = get_homedir(user);
	if (home == 0)
		home = (char *)"";

	n = strlen(home) + strlen(signature_file) + 2;
	n = n > MAXPATHLEN ? MAXPATHLEN : n + 1;
	signature_path = (char *)xmalloc(n);
	#ifdef HAVE_SNPRINTF
	snprintf(signature_path, n - 1,
	#else
	sprintf(signature_path,
	#endif /* HAVE_SNPRINTF */
	"%s/%s", home, signature_file);

	if ((fpsig = fopen(signature_path, "r")) == 0) {
		anubis_error("Signature file error. %s", strerror(errno));
		return;
	}

	while ((n = getc(fpsig)) != EOF)
	{
		if (n == '\n')
			nlines++;
	}
	fseek(fpsig, 0L, SEEK_END);
	clearerr(fpsig);
	nbytes = ftell(fpsig);
	rewind(fpsig);

	buf = (char *)xmalloc(nbytes + nlines + 8);

	strncpy(buf, "\r\n-- \r\n", 7);
	while ((n = getc(fpsig)) != EOF)
	{
		if (n == '\n') /* convert LF -> CRLF */
			strncat(buf, CRLF, 2);
		else if (n == '\r') /* ignore CR */
			n = 0; /* Noop... Do nothing. */
		else {
			char tmp[2];
			tmp[0] = (char)n;
			tmp[1] = '\0';
			strncat(buf, tmp, 1);
		}
	}
	msgbody = (char *)xrealloc((char *)msgbody,
	strlen(msgbody) + strlen(buf) + 1);
	strncat(msgbody, buf, strlen(buf));

	free(buf);
	free(signature_path);
	if (fpsig)
		fpsig = (FILE *)fclose(fpsig);
	return;
}

/*******************************
  Examine file only by suffix.
********************************/

#ifndef TUNNEL_OPTIMIZE
char *examine_file_type(char *file)
{
	#define FILEBUFSIZE 1024
	int c;
	char *ptr;
	char suffix[8];
	unsigned char f_backup[NAME_MAX];
	safe_strcpy(f_backup, file);
	memset(suffix, 0, sizeof(suffix)); 

	/*
	   maybe a config file ?
	*/

	if (f_backup[0] == '.' && f_backup[1] != '.')
		return (char *)mime[1].type; /* text/plain */

	/*
	   else...
	*/

	ptr = strtok((char *)f_backup, ".");
	while ((ptr = strtok(0, ".")) != 0)
		safe_strcpy(suffix, ptr);

	if (suffix[0] == 0) { /* There is NO suffix. */
		int plain_text = 0;
		int bytes = FILEBUFSIZE;
		char buf[FILEBUFSIZE];
		unsigned long nbytes;

		fseek(fp_global, 0L, SEEK_END);
		clearerr(fp_global);
		nbytes = ftell(fp_global);
		rewind(fp_global);

		if (nbytes == 0) /* an empty file */
			return (char *)mime[0].type; /* x-unknown */

		if (nbytes <= FILEBUFSIZE)
			bytes = nbytes;
		fread(buf, 1, bytes, fp_global);

		for (c = 0; c <= bytes; c++)
		{
			if (isprint((unsigned char)buf[c]))
				plain_text++;
		}
		rewind(fp_global);

		if (100 * plain_text / bytes >= 85) /* 85% */
			return (char *)mime[1].type; /* text/plain */
		else
			return (char *)mime[4].type; /* octet-stream */
	}
	else {
		change_to_lower(suffix);
		for (c = 0 ; c < NKEYS_S ; c++)
		{
			if (strcmp(suffix, fsuffix[c].suffix) == 0)
				return (char *)mime[fsuffix[c].id].type;
		}
		return (char *)mime[0].type; /* x-unknown */
	}
}
#endif /* not TUNNEL_OPTIMIZE */

void close_all_files(void)
{
	if (fp_global)
		fp_global = (FILE *)fclose(fp_global);
	if (fp_config)
		fp_config = (FILE *)fclose(fp_config);
	return;
}

/* EOF */

