/*
    misc.c

    This file is a part of Anubis.
    Copyright (C) 2001, 2002 The Anubis Team.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "headers.h"
#include "extern.h"

void set_up_variables(void)
{
	#ifdef NOSSL
	popt |= P_SSL_NO;
	#endif /* NOSSL */

	if (strlen(hostname) == 0)
		safe_strcpy(hostname, DEFAULT_HOSTNAME);

	#ifndef TUNNEL_OPTIMIZE
	if (trans_ptr == 0)
		trans_ptr = uu_base64; /* default Base64 */
	if (popt & P_REALUSER) {
		char host[256];
		#ifndef NOGETHOSTNAME
		gethostname(host, 255);
		#else
		safe_strcpy(host, "localhost");
		#endif /* not GETHOSTNAME */
		xfree(from);
		from = (char *)xmalloc(256);
		mopt |= M_FROM;
		mopt |= M_NOROOT;
		#ifdef HAVE_SNPRINTF
		snprintf(from, 256,
		#else
		sprintf(from,
		#endif /* HAVE_SNPRINTF */
		"%s@%s", anubis_getlogin(), host);
	}
	if ((mopt & M_RM) || (mopt & M_FILEATTACH))
		safe_strcpy(smtp_mode, "mail");
	if ((mopt & M_RMRRT) && (mopt & M_RMPOST))
		anubis_error("Choose only one option: --rm-rrt or --rm-post.");
	#endif /* not TUNNEL_OPTIMIZE */

	return;
}

/*
   An extra port number parser.
*/

void parse_hostname_port(char *opt, char *host, unsigned int *port)
{
	struct servent *p;
	char opt_tmp[128];
	char *port_tmp = 0;
	safe_strcpy(opt_tmp, opt);

	if ((port_tmp = strrchr(opt_tmp, ':'))) {
		*port_tmp++ = '\0';
		if ((p = getservbyname(port_tmp, "tcp")))
			*port = ntohs(p->s_port);
		else
			*port = (unsigned int)atoi(port_tmp);
	}
	strncpy(host, opt_tmp, sizeof(hostname) - 1);
	return;
}

/*
   An extra host name parser.
*/

void parse_hostname_host(char *opt, char *host, unsigned int *port)
{
	struct servent *p;
	char opt_tmp[128];
	char *port_tmp = 0;
	safe_strcpy(opt_tmp, opt);

	if ((port_tmp = strrchr(opt_tmp, ':'))) {
		*port_tmp++ = '\0';
		if ((p = getservbyname(port_tmp, "tcp")))
			*port = ntohs(p->s_port);
		else
			*port = (unsigned int)atoi(port_tmp);
		strncpy(host, opt_tmp, sizeof(tunnel_hostname) - 1);
	}
	else { /* Only a port number is available. */
		if ((p = getservbyname(opt, "tcp")))
			*port = ntohs(p->s_port);
		else
			*port = (unsigned int)atoi(opt);
	}
	return;
}

char *parse_address(char *addr)
{
	char *p = 0;
	char addr_backup[256];
	static char out[256];
	memset(out, 0, sizeof(out));

	if (strstr(addr, "<") && strstr(addr, ">")) { /* name + email address */
		safe_strcpy(addr_backup, addr);
		p = strtok(addr, "<");
		p = strtok(0, "<");
		safe_strcpy(out, p);
		p = strtok(out, ">");
		safe_strcpy(out, p);
		strncpy(addr, addr_backup, strlen(addr_backup));
		return (char *)out;
	}
	else { /* Only an email address. */
		return (char *)addr;
	}
}

/************************
  Remove a single line
*************************/

void remline(char *s, char *line)
{
	char *pos1 = 0;
	char *pos2 = 0;
	int len = 0;

	pos1 = strstr(s, line);
	if (pos1 == 0)
		return;
	pos2 = pos1;
	do {
		pos2++;
	} while (*pos2 != '\n');
	pos2++;
	len = strlen(pos2);
	pos2 = (char *)memmove(pos1, pos2, len);
	pos2[len] = '\0';
	return;
}

void remcrlf(char *s)
{
	int len = strlen(s);

	if (s[len - 2] == '\r' && s[len - 1] == '\n') { /* CRLF */
		s[len - 2] = '\0';
		s[len - 1] = '\0';
	}
	else if (s[len - 2] != '\r' && s[len - 1] == '\n') /* LF */
		s[len - 1] = '\0';
	else if (s[len - 1] == '\r') /* CR */
		s[len - 1] = '\0';
	return;
}

void getoneline(char *s, char *line, int size)
{
	int len = 0;
	char *p = 0;

	memset(line, 0, size);
	p = strchr(s, '\n');
	if (p == 0)
		return;
	p++;
	len = strlen(s) - strlen(p);
	if (len > size)
		len = size;

	strncpy(line, s, len);
	return;
}

void change_to_lower(char *string)
{
	int c;
	for (c = strlen(string) - 1; c >= 0; c--) /* change to lower characters */
		string[c] = tolower((unsigned char)string[c]);
	return;
}

/**************************
  Get a real time & date
***************************/

char *get_time_string(int type)
{
	/*
	   Type: 1 - 'Received:' line; 0 - normal.
	*/

	time_t tp;
	struct tm *ltime;
	static char timebuf[64];

	time(&tp);
	ltime = localtime(&tp);

	if (type == 1) /* for the Received: line */
		strftime(timebuf, sizeof(timebuf), "%a, %d %b %Y %H:%M:%S", ltime);
	else /* standard */
		strftime(timebuf, sizeof(timebuf), "%a, %d %b %Y %H:%M:%S %Z", ltime);

	return (char *)timebuf;
}

/******************
  ROT-13 encoding
*******************/

static void encode_rot13(char *txt)
{
	int c;
	for (c = strlen(txt) - 1; c >= 0; c--)
	{
		txt[c] = (islower((unsigned char)txt[c])
		? 'a'+ (txt[c] - 'a' + 13)%26 : isupper((unsigned char)txt[c])
		? 'A' + (txt[c] - 'A' + 13)%26 : txt[c]);
	}
	return;
}

void check_rot13(void)
{
	if ((mopt & M_SUBJECT) && ((mopt & M_ROT13) || (mopt & M_ROT13S)))
		encode_rot13(subject);
	if ((mopt & M_BODY) && ((mopt & M_ROT13) || (mopt & M_ROT13B)))
		encode_rot13(msgbody);
	return;
}

/*************
   CRAM-MD5
**************/

void cram_md5(char *secret, char *challenge, unsigned char *digest)
{
	MD5_CTX	context;
	unsigned char ipad[64];
	unsigned char opad[64];
	int secret_len;
	int challenge_len;
	int i;

	secret_len = strlen(secret);
	challenge_len = strlen(challenge);
	memset(ipad, 0, sizeof(ipad));
	memset(opad, 0, sizeof(opad));

	if (secret_len > 64) {
		MD5_CTX d_ctx;

		MD5_Init(&d_ctx);
		MD5_Update(&d_ctx, (unsigned char *)secret, secret_len);
		MD5_Final(ipad, &d_ctx);
		MD5_Final(opad, &d_ctx);
	}
	else {
		memcpy(ipad, secret, secret_len);
		memcpy(opad, secret, secret_len);
	}

	for (i = 0; i < 64; i++)
	{
		ipad[i] ^= 0x36;
		opad[i] ^= 0x5c;
	}

	MD5_Init(&context);
	MD5_Update(&context, ipad, 64);
	MD5_Update(&context, (unsigned char *)challenge, challenge_len);
	MD5_Final(digest, &context);

	MD5_Init(&context);
	MD5_Update(&context, opad, 64);
	MD5_Update(&context, digest, 16);
	MD5_Final(digest, &context);

	return;
}

/**********************************
  execargs generator for execvp()
***********************************/

#ifndef NOFORK
char **gen_execargs(const char *commandline)
{
	int argc;
	char *dupstr;
	char *str;
	char **args;

	/* strtok modifies the string. Prevent it by making a private copy. */
	dupstr = strdup(commandline);

	/* Count the arguments first. */
	for (argc = 0, str = strtok(dupstr, " "); str; argc++)
		str = strtok(0, " ");
	free(dupstr);

	args = xmalloc((argc + 1) * sizeof(*args));

	/* strtok above has modified the string, so we duplicate it again. */
	dupstr = strdup(commandline);

	/* Now copy the arguments. */
	for (argc = 0, str = strtok(dupstr, " "); str; argc++)
	{
		args[argc] = strdup(str);
		str = strtok(0, " ");
	}
	args[argc] = 0;
	free(dupstr);

	return args;
}

void free_execargs(char **args)
{
	char **tmp = args;

	while (*tmp)
	{
		free(*tmp);
		tmp++;
	}
	free(args);
	topt &= ~T_RCEXECARGS;
	return;
}
#endif /* not NOFORK */

/* EOF */

