/*
    proxy.c

    This file is a part of Anubis.
    Copyright (C) 2001, 2002 The Anubis Team.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "headers.h"
#include "extern.h"
#include "socks5.h"

static int connect_through_wingate(int, char *, ...);
static int connect_through_socks_proxy(int, char *, unsigned int);

void check_all_proxies(char *host_backup, unsigned int *port_backup)
{
	if ((popt & P_WG2) && !(popt & P_WG1)) {
		safe_strcpy(wingate1, wingate2);
		popt |= P_WG1;
		popt &= ~P_WG2;
	}
	if (popt & P_SOCKS) { /* SOCKS Proxy */
		strncpy(host_backup, socks_proxy, 64);
		*port_backup = socks_port;
	}
	else if ((popt & P_WG1) && !(popt & P_SOCKS)) { /* WinGate 1 (without SOCKS Proxy) */
		strncpy(host_backup, wingate1, 64);
		*port_backup = wingate1_port;
	}
	return;
}

/**********************
   WinGates support
***********************/

int check_wingates(int sd, char *host, unsigned int port)
{
	if (popt & P_WG1) { /* WinGate 1 */
		info(VERBOSE, "Using WinGate(s)...");
		if (popt & P_WG2) { /* WinGate 2 */
			if (connect_through_wingate(sd, "%s %d\r\n", wingate2, wingate2_port) == -1)
				return -1;
		}
		if (connect_through_wingate(sd, "%s %d\r\n", host, port) == -1)
			return -1;
	}
	return 0;
}

static int connect_through_wingate(int sd, char *format, ...)
{
	char buf[LINEBUFFER];
	va_list arglist;

	va_start(arglist, format);
	#ifdef HAVE_VSNPRINTF
	vsnprintf(buf, LINEBUFFER, format, arglist);
	#else
	vsprintf(buf, format, arglist);
	#endif /* HAVE_VSNPRINTF */
	va_end(arglist);

	swrite(CLIENT, (void *)sd, buf);
	info(VERBOSE, "Waiting for a response...");

	/*************************************************
	   We will receive WinGate banner ( WinGate> ),
	   connection status and smtp server banner.
	**************************************************/

	alarm(30);
	while (recvline(CLIENT, (void *)sd, buf, LINEBUFFER))
	{
		change_to_lower(buf);
		if (strstr(buf, "connected")) {
			alarm(0);
			info(VERBOSE, "WinGate: ...connected");
			return 0;
		}
		else
			break;
	}
	anubis_error("WinGate: ...failed");
	return -1;
}

/*************************
   SOCKS Proxy support
**************************/

int check_socks_proxy(int sd, char *host, unsigned int port)
{
	if (popt & P_SOCKS) { /* SOCKS Proxy */
		info(VERBOSE, "Using SOCKS Proxy...");
		if (popt & P_WG1) { /* through WinGate 1 */
			if (connect_through_socks_proxy(sd, wingate1, wingate1_port) == -1)
				return -1;
		}
		else { /* directly */
			if (connect_through_socks_proxy(sd, host, port) == -1)
				return -1;
		}
	}
	return 0;
}

#ifdef IPV6
static int value_of(char what)
{
	switch (what)
	{
		case '0': return 0;
		case '1': return 1;
		case '2': return 2;
		case '3': return 3;
		case '4': return 4;
		case '5': return 5;
		case '6': return 6;
		case '7': return 7;
		case '8': return 8;
		case '9': return 9;
		case 'A': return 10;
		case 'a': return 10;
		case 'B': return 11;
		case 'b': return 11;
		case 'C': return 12;
		case 'c': return 12;
		case 'D': return 13;
		case 'd': return 13;
		case 'E': return 14;
		case 'e': return 14;
		case 'F': return 15;
		case 'f': return 15;
		default: return -1;
	}
}
#endif /* IPV6 */

static int memcopy_offset(char *dst, char *what, int offset)
{
	int i;
	for (i = offset; what[i - offset] != '\0'; i++)
		dst[i] = what[i - offset];
	return i;
}

static int connect_through_socks_proxy(int sd, char *host, unsigned int port)
{
	unsigned char request[1024];
	unsigned char reply[1024];
	int i = 0;
	int ip = 0;
	int offset = 0;

	if (popt & P_SOCKSV4) { /* SOCKS v4 */

		/*
		   Prepare a request.
		*/

		memset(request, 0, sizeof(request));
		request[offset++] = 4;
		request[offset++] = 1;

		/* htons - special edition for SOCKS v4 :-) */
		{
			unsigned char st, nd;
			st = port >> 8;
			nd = (port << 8) >> 8;
			request[offset++] = st;
			request[offset++] = nd;
		}

		for (i = 0; host[i] != '\0'; i++)
		{
			if (isdigit(host[i]) || host[i] == '.')
				ip = 1;
			else
				ip = 0;
		}

		if (ip) {
			unsigned char ip[5];
			unsigned char tmp[5];
			int j = 0, z = 0;

			memset(ip,  0, sizeof(ip));
			memset(tmp, 0, sizeof(tmp));
	
			strncat(host, ".", 1);
			for (i = 0; host[i] != '\0'; i++)
			{
				if (host[i] != '.') {
					tmp[j++] = host[i];
				}
				else {
					j = 0;
					ip[z++] = atoi((char *)tmp);
					memset(tmp, 0, sizeof(tmp));
				}
			}
	
			z = 0;
			for (i = 0; i < 4; i++)
				request[offset++] = ip[z++];
		}
		else {
			socks_proxy_error("Address must be an IP, not a domain name.");
			return -1;
		}

		/*
		   User name ended with...
		*/

		offset = memcopy_offset((char *)request, socks_username, offset);
		request[offset++] = 0x00; /* null */

		/*
		   Send a request.
		*/

		if (send(sd, request, offset, 0) == -1) {
			socket_error();
			return -1;
		}

		/*
		   Get a reply.
		*/

		sleep(1);
		memset(reply, 0, sizeof(reply));
		recv(sd, reply, 8, 0);
    
		/*
		   Process a reply.
		*/

		switch (reply[1])
		{
			case 90:
				info(VERBOSE, "SOCKS Proxy Connection: success");
				break;
			case 91:
				socks_proxy_error("Request rejected or failed.");
				return -1;
			case 92:
				socks_proxy_error("Request rejected.");
				return -1;
			case 93:
				socks_proxy_error("Request rejected, because "
				"the client program and identd reported different User-IDs.");
				return -1;
			default:
				socks_proxy_error("Server reply is not valid.");
				return -1;
		}
	}
	else { /* SOCKS v5 */

		/*
		   Prepare an AUTH request.
		*/

		memset(request, 0, sizeof(request));
		request[offset++] = SOCKS_VERSION;
		request[offset++] = 2;
		request[offset++] = SOCKS_NOAUTH;
		request[offset++] = SOCKS_USERPASS;

		/*
		   Send an AUTH request.
		*/

		if (send(sd, request, offset, 0) == -1) {
			socket_error();
			return -1;
		}

		/*
		   Get a reply.
		*/

		sleep(1);
		memset(reply, 0, sizeof(reply));
		recv(sd, reply, 2, 0);

		/*
		   Check a reply VERSION.
		*/

		if (reply[0] != SOCKS_VERSION) {
			socks_proxy_error("Possibly not a SOCKS proxy service.");
			return -1;
		}

		/*
		   Process an AUTH reply.
		*/

		switch (reply[1])
		{
			case SOCKS_NOAUTH:
				info(VERBOSE,
				"SOCKS Proxy AUTH method: NO AUTHENTICATION REQUIRED");
				break;
			case SOCKS_USERPASS:
				info(VERBOSE,
				"SOCKS Proxy AUTH method: USER NAME/PASSWORD");

				alarm(120);
				if (!(popt & P_SOCKSUSERNAME))
					auth_username(socks_username, sizeof(socks_username));
				if (!(popt & P_SOCKSPASSWORD))
					auth_password("Password: ", socks_password, sizeof(socks_password));
				alarm(0);

				if (!strlen(socks_username) || !strlen(socks_password)) {
					socks_proxy_error("Cannot send null user name or password.");
					return -1;
				}

				/*
				   Prepare User/Pass request.
				*/

				memset(request, 0, sizeof(request));
				offset = 0;
				request[offset++] = SOCKS_PASS_VERSION;
				request[offset++] = strlen(socks_username);
				offset = memcopy_offset((char *)request, socks_username, offset);
				request[offset++] = strlen(socks_password);
				offset = memcopy_offset((char *)request, socks_password, offset);

				/*
				   Send User/Pass request.
				*/

				if (send(sd, request, offset, 0) == -1) {
					socket_error();
					return -1;
				}

				/*
				   Get a reply.
				*/

				sleep(1);
				memset(reply, 0, sizeof (reply));
				recv(sd, reply, 2, 0);

				/*
				   Check a reply.
				*/

				if (reply[1] != SOCKS_USERPASS_OK) {
					socks_proxy_error("Bad user name or password.");
					return -1;
				}
				else
					info(VERBOSE, "SOCKS Proxy AUTH: success");
				break;
			case SOCKS_NAM:
				socks_proxy_error("Server didn`t accept any method.");
				return -1;
			default:
				socks_proxy_error("Server didn't accept an AUTH method.");
				return -1;
		}

		/*
		   Prepare a connection request
		*/

		memset(request, 0, sizeof(request));
		offset = 0;
		request[offset++] = SOCKS_VERSION;
		request[offset++] = SOCKS_CMD_CONNECT;
		request[offset++] = 0;

		for (i = 0; host[i] != '\0'; i++)
		{
			if (isdigit(host[i]) || host[i] == '.') {
				ip = 1; /* IPv4 */
			}
			else
				ip = 0; /* a domain name */

			#ifdef IPV6
			if (host[i] == ':') {
				ip = 2;
				break;
			}
			#endif /* IPV6 */
		}

		if (ip == 1) { /* IPv4 */
			unsigned char ip[5];
			unsigned char tmp[5];
			int j = 0, z = 0;

			memset(ip,  0, sizeof(ip));
			memset(tmp, 0, sizeof(tmp));
			request[offset++] = ATYP_IPv4; /* it`s an IPv4 */

			strncat(host, ".", 1);
			for (i = 0; host[i] != '\0'; i++)
			{
				if (host[i] != '.') {
					tmp[j++] = host[i];
				}
				else {
					j = 0;
					ip[z++] = atoi((char *)tmp);
					memset(tmp, 0, sizeof(tmp));
				}
			}
	
			z = 0;
			for (i = 0; i < 4; i++)
				request[offset++] = ip[z++];

		#ifdef IPV6
		}
		else if (ip == 2) { /* IPv6 */
			char tmp[5];
			int i = 0;

			strncpy(tmp, "0000", 4);
			strncat(host, ":", 1);
			request[offset++] = ATYP_IPv6; /* it`s an IPv6 */

			for (i = 0; host[i] != '\0'; i++)
			{
				if (host[i] != ':') {
					tmp[0] = tmp[1];
					tmp[1] = tmp[2];
					tmp[2] = tmp[3];
					tmp[3] = host[i];
				}
				else {
					request[offset++] = value_of(tmp[0]) * 16 + value_of(tmp[1]);
					request[offset++] = value_of(tmp[2]) * 16 + value_of(tmp[3]);
					strncpy(tmp, "0000", 4);
				}
			}
		#endif /* IPV6 */

		}
		else { /* a domain name */
			request[offset++] = ATYP_DOMAINNAME; /* it`s a domain name */
			request[offset++] = strlen(host);
			offset = memcopy_offset((char *)request, host, offset);
		}

		/* htons - special edition for v5 :-) */
		{
			unsigned char st, nd;
			st = port >> 8;
			nd = (port << 8) >> 8;
			request[offset++] = st;
			request[offset++] = nd;
		}

		/*
		   Send a connection request.
		*/

		if (send(sd, request, offset, 0) == -1) {
			socket_error();
			return -1;
		}

		/*
		   Get a reply.
		*/

		sleep(1);
		memset(reply, 0, sizeof(reply));
		recv(sd, reply, 4, 0); /* We don`t know how long is a reply. */

		/*
		   Process a connection reply.
		*/

		switch (reply[1])
		{
			case 0x00:
				info(VERBOSE, "SOCKS Proxy Connection: success");
				break;
			case 0x01:
				socks_proxy_error("General SOCKS server failure.");
				return -1;
			case 0x02:
				socks_proxy_error("Connection not allowed by ruleset.");
				return -1;
			case 0x03:
				socks_proxy_error("Network unreachable.");
				return -1;
			case 0x04:
				socks_proxy_error("Host unreachable.");
				return -1;
			case 0x05:
				socks_proxy_error("Connection refused.");
				return -1;
			case 0x06:
				socks_proxy_error("TTL expired.");
				return -1;
			case 0x07:
				socks_proxy_error("Command not supported.");
				return -1;
			case 0x08:
				socks_proxy_error("Address type not supported.");
				return -1;
			default:
				socks_proxy_error("Server reply is not valid.");
				return -1;
		}

		switch (reply[3])
		{
			case ATYP_IPv4:
				memset(reply, 0, sizeof(reply));
				/*
				   6 = IPv4 (4) + port number (2)
				*/
				recv(sd, reply, 6, 0);
				break;
			case ATYP_DOMAINNAME:
				{
					int length = 0;
					memset(reply, 0, sizeof(reply));
					recv(sd, reply, 1, 0);
					length = reply[1];
					memset(reply, 0, sizeof(reply));
					recv(sd, reply, length + 2, 0);
				}
				break;

			#ifdef IPV6
			case ATYP_IPv6:
				memset(reply, 0, sizeof(reply));
				/*
				   18 = IPv6 (16) + port number (2)
				*/
				recv(sd, reply, 18, 0);
				break;
			#endif /* IPV6 */
		}
	}
	return 0;
}

/* EOF */

