/*
    quit.c

    This file is a part of Anubis.
    Copyright (C) 2001, 2002 The Anubis Team.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "headers.h"
#include "extern.h"

void sig_exit(int code)
{
	if (output != SILENT)
		putchar('\n');
	info((popt & P_DAEMON) ? NORMAL : VERBOSE,
	"Signal Caught. Exiting Cleanly...");
	quit(code);
}

void sig_timeout(int code)
{
	if (output != SILENT)
		putchar('\n');
	info(NORMAL, "Timeout ! Exiting...");
	quit(code);
}

void free_mem(void)
{
	xfree(msgbody);
	xfree(body_file);
	xfree(rm_rrt);
	xfree(rm_post);
	xfree(rm_header);
	xfree(rm_pgp_file);
	xfree(from);
	xfree(to);
	xfree(reply_to);
	xfree(subject);
	xfree(messageid);
	xfree(xmailer);
	xfree(extra_h1);
	xfree(extra_h2);
	xfree(carbon_copy);
	xfree(blind_carbon_copy);
	xfree(path);
	xfree(filename);
	xfree(fakehost);

	#ifndef NOGPG
	xfree(pgp_keys);
	xfree(rm_pgp_key);
	xfree(pgp_sign_password);
	#endif /* not NOGPG */

	#ifndef NOSSL
	xfree(ssl_cert);
	xfree(ssl_key);
	#endif /* not NOSSL */

	xfree(boundary);
	#ifndef NOFORK
	xfree(execpath);
	if (topt & T_RCEXECARGS)
		free_execargs(execargs);
	#endif /* not NOFORK */

	return;
}

void quit(int code)
{
	#ifndef NOSSL
	end_ssl(CLIENT, ssl_client, ctx_client);
	end_ssl(SERVER, ssl_server, ctx_server);
	ssl_client = 0;
	ssl_server = 0;
	#endif /* not NOSSL */

	#ifndef NOLOGS
	if ((popt & P_DAEMON) && !(popt & P_FOREGROUND))
		closelog();
	#endif /* not NOLOGS */

	close_all_files();
	free_mem();

	exit(code);
}

/* EOF */

